import { EnvContext } from '@typed-macro/core';
import { Macro } from '@typed-macro/core';
import { MacroProvider } from '@typed-macro/core';
import { ModularizedExportable } from '@typed-macro/core';
import { Modules } from '@typed-macro/core';
import { Runtime } from '@typed-macro/runtime';
import { RuntimeOptions } from '@typed-macro/runtime';
import { TransformerOptions } from '@typed-macro/runtime';
import { WatchOptions } from 'chokidar';

/**
 * Create EnvContext for testing.
 *
 * Options will be merged with the default env.
 *
 * Default env:
 * ```typescript
 * {
 *    projectPath: ['/'],
 *    packageManager: 'test',
 *    host: 'test',
 *    dev: true,
 *    ssr: false,
 *    modules: undefined,
 *    watcher: undefined
 * }
 * ```
 */
export declare function createTestEnvContext({ modules, watchOptions, ...env }?: TestEnvContextOptions): EnvContext;

/**
 * Create a runtime to help test providers.
 *
 * e.g.
 * ```typescript
 * const provider = defineMacroProvider(...)
 *
 * const runtime = createTestRuntime({
 *   provider
 * })
 *
 * expect(
 *   runtime.renderTypes()
 * ).toMatchSnapshot()
 *
 * expect(
 *   runtime.load('ns')
 * ).toMatchSnapshot()
 *
 * expect(
 *   runtime.transform(`code`, 'filename')
 * ).toMatchSnapshot()
 *
 * // and so on...
 * ```
 */
export declare function createTestRuntime({ provider, env, ...runtimeOptions }: TestRuntimeOptions): TestRuntime;

/**
 * Create a transformer to help test macros.
 *
 * e.g.
 * ```typescript
 * const aMacro = defineMacro(...)
 *   .withSignature(...)
 *   .withHandler(...)
 *
 * const transform = createTestTransformer({
 *   macros: { '@ns': [aMacro] },
 * })
 *
 * expect(
 *   transform(`some code...`)
 * ).toBe(`some code...`)
 * ```
 */
export declare function createTestTransformer({ maxTraversals, parserPlugins, macros, }: TestTransformerOptions): TestTransformer;

/**
 * Create a type renderer to help test the type of macros/modules.
 *
 * e.g.
 * ```typescript
 * const aMacro = defineMacro(...)
 *   .withSignature(...)
 *   .withHandler(...)
 *
 * const renderType = createTestTypeRenderer()
 *
 * expect(
 *   renderType({
 *     '@ns': {
 *       macros: [aMacro],
 *       types: `some types...`,
 *     },
 *     '@another': {
 *       code: `some code...`,
 *       types: `some types...`,
 *     }
 *   })
 * ).toMatchSnapshot()
 * ```
 */
export declare function createTestTypeRenderer(): TestTypeRenderer;

export declare type TestEnvContextOptions = Partial<Omit<EnvContext, 'watcher' | 'modules'>> & {
    /**
     * Options for EnvContext.watcher.
     *
     * If is undefined, then EnvContext.watcher will be undefined.
     */
    watchOptions?: WatchOptions;
    /**
     * Options for EnvContext.modules.
     *
     * If is true, then EnvContext.modules will be a mocked Modules.
     * If is undefined, then EnvContext.modules will be undefined.
     */
    modules?: boolean | Modules;
};

export declare type TestRuntime = Omit<Runtime, 'renderTypes' | 'appendProvider'> & {
    /**
     * Render types into string.
     */
    renderTypes(): string;
};

export declare type TestRuntimeOptions = RuntimeOptions & {
    /**
     * EnvContext for testing.
     *
     * If is undefined, use the default one created by {@link createTestEnvContext}.
     */
    env?: EnvContext;
    /**
     * Providers to be appended into runtime.
     */
    provider: MacroProvider | MacroProvider[];
};

export declare type TestTransformer = {
    (ctx: TestTransformerContext): string;
    (code: string): string;
};

export declare type TestTransformerContext = {
    /**
     * Source code.
     */
    code: string;
    /**
     * Path of the source file.
     *
     * Default to 'test.ts'
     */
    filepath?: string;
    /**
     * EnvContext for testing.
     *
     * If is undefined, use the default one created by {@link createTestEnvContext}.
     */
    env?: EnvContext;
};

export declare type TestTransformerOptions = TransformerOptions & {
    /**
     * Macros to be appended into transformer.
     */
    macros: {
        [moduleName: string]: Macro[];
    };
};

export declare type TestTypeRenderer = (exports: ModularizedExportable) => string;

export { }
